/*   
 Arduino code for dynamic playseat 3/4DOF v3
 Created 24 May 2011 by Jim Lindblom SparkFun Electronics https://www.sparkfun.com/products/10182 Example Code
 Created 24 Apr 2012 by Jean David SEDRUE Version betatest26 - 24042012 http://www.gamoover.net/Forums/index.php?topic=25907
 Updated 20 Mar 2013 by RacingMat in english http://www.x-sim.de/forum/viewtopic.php?f=37&t=943 in french : http://www.gamoover.net/Forums/index.php?topic=27617
 Updated 4 Aug 2013 by RacingMat for 1 pin direction and 1 pin PWM H-bridges and 3 DOF configuration + Wind :-)
 Updated 30 April 2014 by RacingMat (bug for value below 16 corrected)
 */

#define RV  LOW //beware it's depending on your hardware wiring
#define FW  HIGH //beware it's depending on your hardware wiring

////////////////////////////////////////////////////////////////////////////////
#define pwmMax 255 // or less, if you want to lower the maximum motor's speed

// ratio to amplify pot range (Right or Left) if the pot range is too short (-20пїЅ +20пїЅ for example) 
// too high ratio may amplify jittering (use high quality cermet pot ! and if ou can use mechanical gears between motor and pot) 
#define gearMotorRorLPotRatio 1 
// ratio for the Yaw (third DOF) pot
#define gearMotorYPotRatio 1 

////////////////////////////////////////////////////////////////////////////////
// defining the motors id

#define motLeft 0
#define motRight 1
#define motFrontLeft 2
#define motFrontRight 3
#define motMiddleLeft 4
#define motMiddleRight 5

// defining a shorter authorized range (than 0-1024) for motors along the considered axe
// it's depending on your pot mounting ratio and the max acceptable amplitude of mouvement your sim can handle
const int potMiniRorL=300; 
const int potMaxiRorL=700;
/* these hard-coded values have to be setup without connecting the rods by trial-and-error approach
 once that setup, whatever comes from the simulator software, 
 the arduino will never send over-range value to the motors :-) and your hardware will be protected
 */

const int potMiniY=300;
const int potMaxiY=700;

////////////////////////////////////////////////////////////////////////////////
//  DECLARATIONS
////////////////////////////////////////////////////////////////////////////////
/*  pin definitions*/
//  PWM pin : 3, 5, 6, 9, 10, and 11. Provide 8-bit PWM output with the analogWrite() function.

#define potL A0
#define potR A1
#define potFL A2
#define potFR A3
#define potML A4
#define potMR A5    // no PID needed for Wind -> no Pot on a wind motor

int DIRpin[6] = {
  2 , 4, 7, 8, 12, 13};  // if HIGH/LOW then motor runs Forward/Reverse (depending on wiring)
int PWMpin[6] = {
  3, 5, 6, 9, 10, 11}; // PWM vary between 1 (slow speed) and 255 (fast speed)

/* init position value*/
int DataValueL=512; //middle position
int DataValueR=512; //middle position
int DataValueFL=512; //middle position
int DataValueFR=512; // no Wind at startup
int DataValueML=512; //middle position
int DataValueMR=512; //middle position

////////////////////////////////////////////////////////////////////////////////
// INITIALIZATION
////////////////////////////////////////////////////////////////////////////////
void setup()
{
  // serial initialization
  Serial.begin(115200);

  // initialization of Arduino's pins
  pinMode(potL, INPUT);
  pinMode(potR, INPUT);
  pinMode(potFL, INPUT);
  pinMode(potFR, INPUT);
  pinMode(potML, INPUT);
  pinMode(potMR, INPUT);
  // pinMode(pot4thDOF, INPUT);

  for (int i=0; i<4; i++)
  {
    pinMode(DIRpin[i], OUTPUT);
    pinMode(PWMpin[i], OUTPUT);
  }
  // Initialize braked for motor
  for (int i=0; i<4; i++)
  {
    digitalWrite(PWMpin[i], 0);     
    digitalWrite(DIRpin[i], LOW);
  }

  //  testMotor2(0);

}
////////////////////////////////////////////////////////////////////////////////
///////////////////////////////// Main Loop ////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void loop()
{
  // if the loop is quit, the motors have to be stopped by calling the stop function (if not they would continue to turn independantly of the pot feedback)
  int sensorL,sensorR,sensorFL,sensorFR,sensorML,sensorMR ; // sensorW

  // receiving the game's data
  readSerialData();   // DataValues are stored in a circular buffer
  // If no newer orders are in the serial buffer, the last one is kept
  // And this previous order will still be used by the PID regulation MotorMotion Function

  // reading the pots values

  /* Arduino is reading the pot value through its analogic pin
   pot       O ohm <------> 10 000 ohm
   tension   0 V   <------> 5 V
   pin value 0     <------> 1024
   
   if the mechanical pot's range is too short, increase gearMotorPotRatio (normally =1)*/
  sensorR = constrain(gearMotorRorLPotRatio*(analogRead(potR)-512)+512,0,1024);  // range 0-1024
  sensorL = constrain(gearMotorRorLPotRatio*(analogRead(potL)-512)+512,0,1024);  // range 0-1024
  sensorFL = constrain(gearMotorRorLPotRatio*(analogRead(potFL)-512)+512,0,1024);  // range 0-1024
  sensorFR = constrain(gearMotorRorLPotRatio*(analogRead(potFR)-512)+512,0,1024);  // range 0-1024
  sensorML = constrain(gearMotorRorLPotRatio*(analogRead(potML)-512)+512,0,1024);  // range 0-1024
  sensorMR = constrain(gearMotorRorLPotRatio*(analogRead(potMR)-512)+512,0,1024);  // range 0-1024

  //  sensorW = analogRead(potW);  // range 0-1024 

    motorMotion(motRight,sensorR,DataValueR, potMiniRorL,potMaxiRorL);
   motorMotion(motLeft,sensorL,DataValueL,potMiniRorL,potMaxiRorL);
  motorMotion(motFrontLeft,sensorFL,DataValueFL,potMiniRorL,potMaxiRorL);
  motorMotion(motFrontRight,sensorFR,DataValueFR,potMiniRorL,potMaxiRorL);
  motorMotion(motMiddleLeft,sensorML,DataValueML,potMiniRorL,potMaxiRorL);
  motorMotion(motMiddleRight,sensorMR,DataValueMR,potMiniRorL,potMaxiRorL);
  //  motorMotion(motWind,0,DataValueW,0,1024); 
  //  motorMotion(mot4thDOF,sensor4thDOF,DataValue4thDOF,potMini4thDOF, PotMaxi4thDOF); // in case it's used for 4th DOF and not wind motor

}
////////////////////////////////////////////////////////////////////////////////
// Procedure: wait for complete trame
////////////////////////////////////////////////////////////////////////////////
void readSerialData()
{
  byte Data[3]={
    '0','0','0'                            };
  // keep this function short, because the loop has to be short to keep the control over the motors

  // if (Serial.available()>=(2*SignalLength-1)) {
  //we are sure to have a whole signal to decode inside the buffer
  // and no more than one, in order to have the newest (latest) as possible


  if (Serial.available()>2){ //wait a packet of 6
    //parse the buffer : test if the byte is the first of the order R
    Data[0]=Serial.read();
    if (Data[0]=='R'){
      Data[1]=Serial.read();
      Data[2]=Serial.read();
      //  call the function that converts the hexa in decimal and maps the range
      // map the range from Xsim (0-255) to the mechanically authorized range (potMini-potMaxi)
      DataValueR=map(NormalizeData(Data),0,255,potMiniRorL,potMaxiRorL);
    }
    if (Data[0]=='L'){
      Data[1]=Serial.read();
      Data[2]=Serial.read();
      //  call the function that converts the hexa in decimal and maps the range
      DataValueL=map(NormalizeData(Data),0,255,potMiniRorL,potMaxiRorL);
    }
    if (Data[0]=='FL'){
      Data[1]=Serial.read();
      Data[2]=Serial.read();
      //  call the function that converts the hexa in decimal and maps the range
      DataValueFL=map(NormalizeData(Data),0,255,potMiniRorL,potMaxiRorL);
    }
    if (Data[0]=='FR'){
      Data[1]=Serial.read();
      Data[2]=Serial.read();
      //  call the function that converts the hexa in decimal and maps the range
      DataValueFR=map(NormalizeData(Data),0,255,potMiniRorL,potMaxiRorL);
    }
    if (Data[0]=='ML'){
      Data[1]=Serial.read();
      Data[2]=Serial.read();
      //  call the function that converts the hexa in decimal and maps the range
      DataValueML=map(NormalizeData(Data),0,255,potMiniRorL,potMaxiRorL);
    }
    if (Data[0]=='MR'){
      Data[1]=Serial.read();
      Data[2]=Serial.read();
      //  call the function that converts the hexa in decimal and maps the range
      DataValueMR=map(NormalizeData(Data),0,255,potMiniRorL,potMaxiRorL);
    }
  }
  if (Serial.available()>16) Serial.flush();
  // no need to flush because reading the bytes has reduced the serial buffer and it will be filled in by the next incoming data
}
////////////////////////////////////////////////////////
void motorMotion(int numMot,int actualPos,int targetPos,int potMini, int potMaxi )
////////////////////////////////////////////////////////
{
  int Tol=20;
  //could be a parameter read from a pot on an analogic pin
  // the highest, the calmest the simulator would be (less moves)
  // % (PotMaxi-potMini)
  int gap;
  int pwm;
  int brakingDistance=30;

  // security concern : targetPos has to be within the mechanically authorized range
  targetPos=constrain(targetPos,potMini+brakingDistance,potMaxi-brakingDistance);

  gap=abs(targetPos-actualPos);

  if (gap<= Tol) {
    motorOff(numMot); //too near to move     
  }
  else {
    // PID : calculate speed proportionaly to distance to go
    pwm=195;
    if (gap>50)   pwm=215;
    if (gap>75)   pwm=235;   
    if (gap>100)  pwm=255;

    pwm=map(pwm, 0, 255, 0, pwmMax);  //adjust the value according to pwmMax for mechanical debugging purpose !

    // pwm=155;  // debug

    // if motor is outside from the range, send motor back to the limit !
    // go forward (up)
    if ((actualPos<potMini) || (actualPos<targetPos)) motorGo(numMot, FW, pwm);
    // go reverse (down)   
    if ((actualPos>potMaxi) || (actualPos>targetPos)) motorGo(numMot, RV, pwm);

  }
}

////////////////////////////////////////////////////////////////////////////////
void motorOff(int motor){ //Brake //Ground : free wheel actually
  ////////////////////////////////////////////////////////////////////////////////
  analogWrite(PWMpin[motor], 0);
}

////////////////////////////////////////////////////////////////////////////////
void motorGo(int motor, int directionOfRotation, int pwm)
////////////////////////////////////////////////////////////////////////////////
{
  digitalWrite(DIRpin[motor], directionOfRotation);
  analogWrite(PWMpin[motor], pwm);
}


////////////////////////////////////////////////////////////////////////////////
// Function: convert Hex to Dec
////////////////////////////////////////////////////////////////////////////////
int NormalizeData(byte x[3])
////////////////////////////////////////////////////////////////////////////////
{
  int result;

  if ((x[2]==13) || (x[2]=='R') || (x[2]=='L'))  //only a LSB and Carrier Return or 'L' or 'R' in case of value below 16 (ie one CHAR and not 2)
  {
    x[2]=x[1];  //move MSB to LSB
    x[1]='0';     //clear MSB
  }
  for (int i=1; i<3; i++)
  {
    if (x[i]>47 && x[i]<58 ){//for xA to xF
      x[i]=x[i]-48;
    }                       
    if (  x[i]>64 && x[i]<71 ){//for x0 to x9
      x[i]=x[i]-55;               
    }
  }
  result=(x[1]*16+x[2]);
  return result;
}
////////////////////////////////////////////////////////////////////////////////
// Function: convert Hex to Dec
////////////////////////////////////////////////////////////////////////////////

int HexaToDecimal(byte x[2])
{
  byte ret = 0, t = 0;
  for (int i=0; i<2; i++)
  {
    if((x[i] >= '0') && (x[i] <= '9')) t = (x[i] - '0');
    else if((x[i] >= 'A') && (x[i] <= 'F')) t = (x[i] - 'A' + 10);
    else if((x[i] >= 'a') && (x[i] <= 'f')) t = (x[i] - 'a' + 10);
    else break;
    ret *= 16;
    ret += t;
  }
  return ret;
}

////////////////////////////////////////////////////////////////////////////////
void testMotor2(int numMot){
  motorOff(numMot);
  delay(1000);
  motorGo(numMot, FW, 55);
  delay(1000);
  motorOff(numMot);
  delay(1000);
  motorGo(numMot, RV, 55);
  delay(1000);
  motorOff(numMot);
  delay(1000);
  motorGo(numMot, FW, 55);
  delay(1000);
  motorOff(numMot);
  delay(1000);
  motorGo(numMot, RV, 55);
  delay(1000);
  motorOff(numMot);
  delay(1000);

}


////////////////////////////////////////////////////////////////////////////////
void testPotToSerial(){

  Serial.print(analogRead(A4));
  Serial.print(";");
  Serial.println(analogRead(A5));
  delay(250);
}


